module DictSw where

import BuildSw
import RulesSw
import General
import Dictionary

swedishDict :: Dictionary
swedishDict = dictionary $ closedLexicon ++ hardVerbs ++
 [
  substUtrum (decl2mor "mor"),
  --substUtrum (decl2mor "mormor"),
  substUtrum (decl2far "far"),
  --substUtrum (decl2far "farfar"),
  --substUtrum (decl2far "morfar"),
  substUtrum (decl2far "bror"),
  --substUtrum (decl2far "farbror"),
  --substUtrum (decl2far "morbror"),
  sNyckel "syster",
  sNyckel "faster",
  sNyckel "moster",
  aReg "amerikansk",
  aReg "apatisk",
  aAbstrakt "arrogant",
  adjective (mkAdjective 
     "gammal" "gammalt" "gamla" "gamla" "ldre" "ldst" "ldsta"),
  v1 "hvda",
  s3 "imperialism",
  s3 "imperialist",
  aReg "jmn",
  s3 "kapitalist",
  aReg "klok",
  aReg "mjlig",
  substUtrum (mkSubstantive "son" "sonen" "sner" "snerna" nonExist),
  adjective (adjIrreg3 "stor" "strre" "strst"),
  v2 "tycka",
  aReg "tnkbar",
  aUdda "udda",
  adjective (adjIrreg3 "ung" "strre" "strst"),
  aReg "vertikal",
  verb (mkVerb "veta" "vet" "vete" "vet" "visste" "visste" "vetat" "visst") ,
-- models for the most common cases

  vStrong "stjla" "stal" "stulit",

  pnMasc "Oden",
  pnUtr "Sigyn",
  pnNeutr "Sverige",
  
  advReg  "fint",
  adverbBra "bra" "bttre" "bst",
  adverbBra "grna" "hellre" "helst",
  adverb (advAdj (adjFager "vacker")),
  adverbInv "mjligen",
  adverbInv "kanske",
  adverbInv "aldrig",
  adverbInv "alltid",
  
-- interrogatives
  interrogInv "nr",
  interrogInv "hur",
  interrogInv "vad",
  interrogInv "varfr",
  interrogInv "vart",
  interrogInv "var",
  interrogInv "varifrn",

  pronInv "man",

-- ambiguities
  s2 "val",
  s5 "val",
  s2 "vals",
  s3 "vals",

-- adjectives

-- the most irregular words

  substMasc (sIrreg "man" "mannen" "mn" "mnnen" nonExist),
  substUtrum (sIrreg "gs" "gsen" "gss" "gssen" nonExist),
  adjective (aIrreg "liten" "litet" "lilla" "sm" "mindre" "minst" "minsta"),
  verb (noPartPret (vIrreg "f" "fr" "f" "f" "fick" "finge" "ftt" [])),
  verb (noPassive (vIrreg "vara" "r" "vare" "var" "var" "vore" "varit" []))
  ]

hardVerbs :: [Entry]
hardVerbs =  concat [
  verbs ["","an","av","be","in","om","upp","ut","ter","ver"] 
    (shortLong "ge" (verbStrongGiva "giva")),
  verbs ["","fr","ute","ver"] 
    (noPassive (shortLong "bli" (verbStrongSmita "bliva"))),
  verbs ["","an","av","be","fr","in","om","upp","ut","ter","ver"] 
     (vIrreg "se" "ser" "se" "se" "sg" "sg" "sett" "sedd"),
  verbs ["","an","av","be","frn","fr","genom","in","kring","upp","ut","ver"] 
      (vIrreg "g" "gr" "g" "g" "gick" "ginge" "gtt" "gngen"),
  verbs ["","an","av","be","fr","genom","in","upp","ut","ter"] 
      (vIrreg "st" "str" "st" "st" "stod" "stnde" "sttt" "stnden"),
  verbs ["","inne","ter"] 
      (noPartPret (shortLong "ha" 
          (vIrreg "hava" "haver" "have" "hav" "hade" "hade" "haft" [])))
  ]


closedLexicon :: [Entry]
closedLexicon = concat 
		[
  		 prepositions,
                 adverbials,		 
   		 conjunctions,
   		 subjunctions,
   		 particles,
		 pronounsPN,
		 pronounsAdj,
		 [
		  entry artIndef,
		  entry artDef,
		  infMark "att"
		 ],
                 auxiliaryVerbs,
		 numerals 
		]

pronounsAdj = 
    [
     pronDylik "all",
     pronDylik "dylik",
     pronDylik "hurdan",
     pronDylik "likadan",
     pronDylik "sdan",
     pronNagon "annan" "annat" "andra",
     pronNagon "ingen" "inget" "inga",
     pronNagon "vilken" "vilket" "vilka",
     pronNagon "mngen" "mnget" "mnga",
     pronNagon "ngon" "ngot" "ngra"
    ]


prepositions 
     = map prep
	[
       "alltsedan",
       "aprop",
       "av",
       "bakom",
       "bland",
       "bortom",
       "bortt",
       "brevid",
       "efter",
       "emellan",
       "emot",
       "enligt",
       "framfr",
       "frn",
       "fr",
       "frbi",
       "fre",
       "frutom",
       "genom",
       "gentemot",
       "hos",
       "hrom",
       "i",
       "ifrn",
       "igenom",
       "inemot",
       "infr",
       "inklusive",
       "innan",
       "innanfr",
       "inom",
       "intill",
       "invid",
       "jmte",
       "kontra",
       "kring",
       "lngs",
       "med",
       "medelst",
       "mellan",
       "mot",
       "nedanfr",       
       "om",
       "omkring",
       "ovan",
       "ovanfr",
       "ovanp",
       "per",
       "p",
       "runt",
       "sedan",
       "sen",
       "till",
       "trots",
       "undan",
       "under",
       "ur",
       "utan",
       "utanfr",
       "utanp", 
       "utefter",
       "utfr",
       "utifrn",
       "utmed",
       "utom",
       "utver",
       "via",
       "vid",
       "visavi",
       "", 
       "t",
       "ver"
       ]

conjunctions = map conj $ words	
  "och samt respektive liksom fast men utan eller ty fr s"

subjunctions = map subj $ words
	       "att frrn innan medan sedan tills eftersom allteftersom emedan fr s bara blott ifall om svida liksom som ssom n ehuru fast fastn oaktat"

particles = map part $ words
   "emot fast in ut upp ner fram ter" -- plus prepositions

auxiliaryVerbs = map (\ (x, y, z, u) -> entry (auxVerbGen x y z u)) [
  (mkStr "vilja", mkStr "vill", mkStr "ville",mkStr "velat"),
  (mkStr "kunna", mkStr "kan",  mkStr "kunde",mkStr "kunnat"),
  (nonExist,      mkStr "mste",mkStr "mste",mkStr "mst"),
  (mkStr "skola", strings ["ska","skall"], mkStr "skulle",nonExist),
  (mkStr "bra",  mkStr "br", mkStr "borde",mkStr "bort")
  ]

-- -- not really closed
adverbials = map adverbInv $ words 
  "dessutom heller ocks ven dock nd allts vl nere ute inne"

-- from numerals.Swe.gf
numerals = map numeralInv $ words "arton aderton elva ett fem femtio femton fjorton fyra fyrtio hundra nio nittio nitton sex sextio sexton sju sjuttio sjutton tio tjugo tolv tre trettio tretton tusen tv tta ttio"